AWSCONF_SHEET = 'contentfiles2aws'
PROJECTNAME = 'collective.contentfiles2aws'

ADD_PERMISSIONS = {
    'AWSFile':'collective.contentfiles2aws: Add AWSFile',
    'AWSImage':'collective.contentfiles2aws: Add AWSImage',
}

SKIP_SOURCE_REMOVE_FLAG = 'collective.contentfiles2aws.skip_aws_source_remove'
ABORT_TRANSACTION_FLAG = 'collective.contentfiles2aws.abort_transaction'
